// compiler avec :  g++ -I. jsoncpp.cpp exjson.cpp  -o exjson -lm
#include <iostream>
#include <fstream>
#include <stdexcept>
#include <cmath>
#include <json/json.h>


/*--------------------------------------------------------------------------*/
// classe de parametres
/*--------------------------------------------------------------------------*/
class Tparam
{
    public:
        // constructeur
        Tparam() : npas(-1), t0(std::nan(""))  {}
        
        // lecture du fichier de parametres
        void read(const std::string& filename);
        
        // ecriture du fichier de parametres
        void write(const std::string& filename) const;
        
        // dump du contenu de la classe
        void dump(std::ostream& of) const;
        
        // nom du fichier de sortie
        const std::string& getfichiersortie() { return fichiersortie; }
   protected:
       // donnees
       std::string fichiersortie;
       int         npas;
       double      t0;      
       std::vector<double> angles; 
};

/*--------------------------------------------------------------------------*/
// Lecture des parametres
/*--------------------------------------------------------------------------*/
void Tparam::read(const std::string& filename)
{
    std::cout << "lecture du fichier " << filename << std::endl;
    std::ifstream ifs(filename);
    Json::Value root; 
    ifs >> root;
    fichiersortie = root.get("fichiersortie","invalidfile").asString();
    npas          = root.get("npas",-1).asInt();
    t0            = root.get("t0", std::nan("")).asDouble();
    
    Json::Value jsonangles = root["angles"];
    for(Json::Value::iterator it=jsonangles.begin(); it!=jsonangles.end(); ++it)
    {
        angles.push_back(it->asDouble());
    }
}

/*--------------------------------------------------------------------------*/
// Ecriture des parametres
/*--------------------------------------------------------------------------*/
void Tparam::write(const std::string& filename) const
{
    std::cout <<"ecriture du fichier "<< filename << std::endl;
    Json::Value root;    
    root["fichiersortie"] = fichiersortie;
    root["npas"]          = npas;
    root["t0"]            = t0;
    
    Json::Value jsonangles;
    for(std::vector<double>::const_iterator it=angles.begin(); it!=angles.end(); ++it)
    {
        jsonangles.append(*it);
    }
    root["angles"]       = jsonangles;
    
    std::ofstream ofs(filename);
    ofs << root << std::endl;
}

/*--------------------------------------------------------------------------*/
// Dump du contenu de la classe 
/*--------------------------------------------------------------------------*/
void Tparam::dump(std::ostream& of) const
{
    of<<"Tparam = "<<std::endl;    
    of<<"  fichiersortie = '"<<fichiersortie<<"'"<<std::endl;    
    of<<"  npas          = "<<npas<<std::endl;    
    of<<"  t0            = "<<t0<<std::endl;    
    of<<"  angles        = [";
    for(std::vector<double>::const_iterator it=angles.begin(); it!=angles.end(); ++it)
    {
        of <<*it<<", ";
    }
    of<<"]"<<std::endl;    
}

/*--------------------------------------------------------------------------*/
//programme principal
/*--------------------------------------------------------------------------*/
int main(int argc, char *argv[])
{
    if (argc!=2) throw std::runtime_error("usage : exjson input.json");
    std::string fichierentree(argv[1]);
    Tparam myparam;
    myparam.read(fichierentree);
    myparam.dump(std::cout);
    myparam.write(myparam.getfichiersortie()); 
}

