!----------------------------------------------------------------------------------------
! exemple: fichier de parametres utilisant un fichier json
! compiltion :  gfortran -Jlib exjson.f90 -o exjson -Llib -ljsonfortran
!----------------------------------------------------------------------------------------
module mparam

  type tparam
  character (len=:), allocatable :: fichiersortie
  integer :: npas
  real(kind=8) :: t0
  real(kind=8), dimension(:), allocatable :: angles
end type

contains

!----------------------------------------------------------------------------------------
! lecture du fichier de parametres : fichierparam
! stocke les valeurs lues dans mesparams
!----------------------------------------------------------------------------------------
subroutine readparam(fichierparam, mesparams)
  use json_module
  implicit none
  character(len=*), intent(in) :: fichierparam
  type(tparam), intent(out) :: mesparams

  type(json_file) :: json
  logical :: found

  write(*,*) 'lecture du fichier ', trim(fichierparam)
  
  call json%initialize()
  call json%load_file(filename=fichierparam)
  
  call json%get('fichiersortie', mesparams%fichiersortie, found)
  call json%get('npas',          mesparams%npas,          found)
  call json%get('t0',            mesparams%t0,            found)
  call json%get('angles',       mesparams%angles,         found)
  return
end subroutine readparam

!----------------------------------------------------------------------------------------
! lecture du fichier de parametres : fichierparam
! stocke les valeurs lues dans mesparams
!----------------------------------------------------------------------------------------
subroutine writeparam(fichierparam, mesparams)
  use json_module
  implicit none
  character(len=*), intent(in) :: fichierparam
  type(tparam), intent(in) :: mesparams

  type(json_file) :: json

  write(*,*) 'ecriture du namelist ', trim(fichierparam)
  
  call json%initialize(compact_reals=.false.)
  call json%add('fichiersortie', mesparams%fichiersortie)
  call json%add('npas',          mesparams%npas)
  call json%add('t0',            mesparams%t0)
  call json%add('angles',       mesparams%angles)
  
  call json%print_file(fichierparam)

  return
end subroutine writeparam

end module mparam

!-----------------------------------------------------------------------------------
! retourne le fichier de parametres a utiliser en inspectant la ligne de commandes
! pour les appels du type :
!
!       monapplication monfichierparam.par
!
!  en cas d'erreur ou nombre d'arguments !=1 : arret avec message d'erreur
!
!
!  @param parfilename (out) nom du fichier de parametre retourne
!
!
!-----------------------------------------------------------------------------------
subroutine getparfilename(parfilename)
  implicit none
  character(len=*), intent(out)  :: parfilename

  integer status, length
  integer cmd_count
  logical fatalerror

  fatalerror = .false.

  ! recupere le nombre d'arguments
  cmd_count = COMMAND_ARGUMENT_COUNT()

  if (cmd_count .eq. 1) then
    ! recupere le 1er argument
    call GET_COMMAND_ARGUMENT(1, parfilename, length, status)
    if (status .ne. 0) then
      fatalerror = .true.
    endif
  else
    fatalerror = .true.
  endif

  if (fatalerror .eqv. .true.) then
    write (*, *)
    write (*, *) "Usage : <nom du programme> <fichierparam.nml>"
    stop
  endif

  return
end

!----------------------------------------------------------------------------------------
! programme principal
!----------------------------------------------------------------------------------------
program paramnml
  use mparam
  implicit none
  type(tparam) :: mesparams
  character(len=512) :: nomfichierparam

  call getparfilename(nomfichierparam)
  call readparam(nomfichierparam, mesparams)
  call writeparam(mesparams%fichiersortie, mesparams)

  stop
end
