!----------------------------------------------------------------------------------------
! exemple: fichier de parametres utilisant un namelist
!----------------------------------------------------------------------------------------
module mparam

  type tparam
  character(len=512) :: fichiersortie
  integer :: npas
  real(kind=8) :: t0
  real(kind=8), dimension(:), allocatable :: angles
end type

contains

!----------------------------------------------------------------------------------------
! lecture du fichier de parametres : fichierparam
! stocke les valeurs lues dans mesparams
!----------------------------------------------------------------------------------------
subroutine readparam(fichierparam, mesparams)
  implicit none
  character(len=*), intent(in) :: fichierparam
  type(tparam), intent(out) :: mesparams

  character(len=512) :: fichiersortie
  integer :: npas
  real(kind=8) :: t0
  real(kind=8), dimension(1:3) :: angles 
  ! definition du namelist
  namelist /mesinputs/ fichiersortie, npas, t0, angles

  write(*,*) 'lecture du namelist ', trim(fichierparam)
  ! lecture du namelist
  open  (10, file=fichierparam)
  read  (10, mesinputs)
  close (10)

  ! stockage dans le type derive
  mesparams%fichiersortie = fichiersortie
  mesparams%npas          = npas
  mesparams%t0            = t0
  allocate(mesparams%angles(lbound(angles,1):ubound(angles,1)), source = angles)
  return
end subroutine readparam

!----------------------------------------------------------------------------------------
! lecture du fichier de parametres : fichierparam
! stocke les valeurs lues dans mesparams
!----------------------------------------------------------------------------------------
subroutine writeparam(fichierparam, mesparams)
  implicit none
  character(len=*), intent(in) :: fichierparam
  type(tparam), intent(in) :: mesparams

  character(len=512) :: fichiersortie
  integer :: npas
  real(kind=8) :: t0
  real(kind=8), dimension(:), allocatable :: angles
  
  ! definition du namelist
  namelist /mesinputs/ fichiersortie, npas, t0, angles

  write(*,*) 'ecriture du namelist ', trim(fichierparam)
  ! recuperation des donnees du type derive
  fichiersortie = mesparams%fichiersortie
  npas          = mesparams%npas 
  t0            = mesparams%t0
  allocate(angles(lbound(mesparams%angles,1):ubound(mesparams%angles,1)), source= mesparams%angles)
  ! ecriture du namelist
  open  (10, file=fichierparam, status='unknown', DELIM='QUOTE')
  write (10,  mesinputs) 
  close (10)

  return
end subroutine writeparam

end module mparam

!-----------------------------------------------------------------------------------
! retourne le fichier de parametres a utiliser en inspectant la ligne de commandes
! pour les appels du type :
!
!       monapplication monfichierparam.par
!
!  en cas d'erreur ou nombre d'arguments !=1 : arret avec message d'erreur
!
!
!  @param parfilename (out) nom du fichier de parametre retourne
!
!
!-----------------------------------------------------------------------------------
subroutine getparfilename(parfilename)
  implicit none
  character(len=*), intent(out)  :: parfilename

  integer status, length
  integer cmd_count
  logical fatalerror

  fatalerror = .false.

  ! recupere le nombre d'arguments
  cmd_count = COMMAND_ARGUMENT_COUNT()

  if (cmd_count .eq. 1) then
    ! recupere le 1er argument
    call GET_COMMAND_ARGUMENT(1, parfilename, length, status)
    if (status .ne. 0) then
      fatalerror = .true.
    endif
  else
    fatalerror = .true.
  endif

  if (fatalerror .eqv. .true.) then
    write (*, *)
    write (*, *) "Usage : <nom du programme> <fichierparam.nml>"
    stop
  endif

  return
end

!----------------------------------------------------------------------------------------
! programme principal
!----------------------------------------------------------------------------------------
program paramnml
  use mparam
  implicit none
  type(tparam) :: mesparams
  character(len=512) :: nomfichierparam

  call getparfilename(nomfichierparam)
  call readparam(nomfichierparam, mesparams)
  call writeparam(trim(mesparams%fichiersortie), mesparams)

  stop
end
